@@ -342,7 +342,7 @@ void sstp_log_usage(void)
     printf("  --log-syslog <sock>      Output to syslog\n");
     printf("  --log-stderr             Output to stderr (negates --log-stdout)\n");
     printf("  --log-stdout             Output to stdout (negates --log-stderr)\n");
-    printf("  --log-lineno             Include file/line information in messags\n");
+    printf("  --log-lineno             Include file/line information in messages\n");
     printf("  --log-ident              Specify log identity\n");
     printf("  --log-filter <tok,tok>   Log messages matching a token\n\n");
 }
