@@ -104,6 +104,10 @@ static void sstp_client_pppd_cb(sstp_client_st *client, sstp_pppd_event_t ev)
 
     switch (ev)
     {
+    case SSTP_PPP_START:
+        sstp_state_resume_recv(client->state);
+        break;
+
     case SSTP_PPP_DOWN:
         log_err("PPPd terminated");
         //sstp_state_disconnect(client->state);
@@ -150,9 +154,10 @@ static void sstp_client_pppd_cb(sstp_client_st *client, sstp_pppd_event_t ev)
 /*!
  * @brief Called when the state machine transitions
  */
-static void sstp_client_state_cb(sstp_client_st *client, sstp_state_t event)
+static status_t sstp_client_state_cb(void *arg, sstp_state_t event)
 {
-    int ret = 0;
+    sstp_client_st *client = (sstp_client_st*) arg;
+    status_t ret = SSTP_OKAY;
 
     switch (event)
     {
@@ -169,7 +174,7 @@ static void sstp_client_state_cb(sstp_client_st *client, sstp_state_t event)
         /* Start the pppd daemon */
         ret = sstp_pppd_start(client->pppd, &client->option, 
                 sstp_event_sockname(client->event));
-        if (SSTP_OKAY != ret)
+        if (SSTP_FAIL == ret)
         {
             sstp_die("Could not start PPP daemon", -1);
         }
@@ -179,7 +184,7 @@ static void sstp_client_state_cb(sstp_client_st *client, sstp_state_t event)
                 sstp_pppd_send, client->pppd);
 
         log_info("Started PPP Link Negotiation");
-        break;
+        return ret;
     
     case SSTP_CALL_ESTABLISHED:
 
@@ -196,20 +201,19 @@ static void sstp_client_state_cb(sstp_client_st *client, sstp_state_t event)
                 log_warn("Could not enter privilege directory");
             }
         }
-
         break;
 
     case SSTP_CALL_ABORT:
     default:
 
-	if (client->pppd) 
+        if (client->pppd)
         {
-	    sstp_pppd_stop(client->pppd);
+	        sstp_pppd_stop(client->pppd);
         }
-        sstp_die("Connection was aborted, %s", -1, 
+        sstp_die("Connection was aborted, %s", -1,
                 sstp_state_reason(client->state));
-        break;
     }
+    return ret;
 }
 
 
@@ -248,7 +252,7 @@ static void sstp_client_http_done(sstp_client_st *client, int status)
     }
 
     /* Now we need to start the state-machine */
-    status = sstp_state_create(&client->state, client->stream, (sstp_state_change_fn)
+    status = sstp_state_create(&client->state, client->stream,
             sstp_client_state_cb, client, SSTP_MODE_CLIENT);
     if (SSTP_OKAY != status)
     {
