@@ -62,8 +62,8 @@ struct sstp_pppd
 
     /*< The notify function */
     sstp_pppd_fn notify;
-    
-    /*< The argument to pass to this function */
+
+    /*< The argument to pass to the notify function */
     void *arg;
 
     /*< The socket to pppd */
@@ -95,6 +95,9 @@ struct sstp_pppd
 
     /*< The number of bytes received */
     unsigned long long recv_bytes;
+
+    /*< First read from pppd */
+    bool first;
 };
 
 
@@ -133,7 +136,7 @@ static void ppp_send_complete(sstp_stream_st *stream, sstp_buff_st *buf,
 {
     if (SSTP_OKAY != status)
     {
-        log_err("TODO: Handle shutdown here");
+        log_err("Failed to complete write to pppd");
     }
 
     /* Continue processing input */
@@ -154,7 +157,7 @@ static void ppp_send_complete(sstp_stream_st *stream, sstp_buff_st *buf,
 
     case SSTP_FAIL:
     default:
-        log_err("TODO: Handle processing failure");
+        log_err("Failed to receive data from pppd");
         break;
     }
 }
@@ -351,7 +354,7 @@ static status_t ppp_process_data(sstp_pppd_st *ctx)
         {
             /* We needed to read more ... */
             if (SSTP_OVERFLOW == ret ||
-               (rx->len == (rx->off + off))) // TODO: Why!?!
+               (rx->len == (rx->off + off)))
             {
                 /* Move current packet to beginning of buffer */
                 memmove(rx->data, rx->data + rx->off, rx->len - rx->off);
@@ -419,6 +422,12 @@ static void sstp_pppd_recv(int fd, short event, sstp_pppd_st *ctx)
     sstp_buff_st *rx = ctx->rx_buf;
     status_t ret = SSTP_FAIL;
 
+    /* Notifiy the caller pppd is configured and running */
+    if (ctx->first) {
+        ctx->notify(ctx->arg, SSTP_PPP_START);
+        ctx->first = false;
+    }
+
     /* Receive a chunk */
     rx->len += read(fd, rx->data + rx->len, rx->max - rx->len);
     if (rx->len <= 0)
@@ -445,7 +454,7 @@ static void sstp_pppd_recv(int fd, short event, sstp_pppd_st *ctx)
 
     case SSTP_FAIL:
     default:
-        log_err("TODO: Handle failure of processing");
+        log_err("Failed to receive data from pppd");
         break;
     }
 
@@ -503,7 +512,7 @@ done:
 }
 
 
-status_t sstp_pppd_start(sstp_pppd_st *ctx, sstp_option_st *opts, 
+status_t sstp_pppd_start(sstp_pppd_st *ctx, sstp_option_st *opts,
         const char *sockname)
 {
     status_t status  = SSTP_FAIL;
@@ -597,12 +606,18 @@ status_t sstp_pppd_start(sstp_pppd_st *ctx, sstp_option_st *opts,
         }
 
         /* Get the socket to listen on */
-        ctx->sock    = sstp_task_stdout(ctx->task);
+        ctx->sock = sstp_task_stdout(ctx->task);
+
+        /* Wait until pppd is configured and running before resuming recv() on SSTP stream */
+        status = SSTP_INPROG;
     }
     else
     {
         /* pppd is our parent, we communciate over a pty terminal */
         ctx->sock = STDIN_FILENO;
+
+        /* Success! */
+        status = SSTP_OKAY;
     }
 
     /* Need to record approximate time */
@@ -615,12 +630,9 @@ status_t sstp_pppd_start(sstp_pppd_st *ctx, sstp_option_st *opts,
     /* Add the receive event */
     event_add(ctx->ev_recv, NULL);
 
-    /* Success! */
-    status = SSTP_OKAY;
-
 done:
 
-    return (status);
+    return status;
 }
 
 
@@ -674,6 +686,7 @@ status_t sstp_pppd_create(sstp_pppd_st **ctx, event_base_st *base,
     (*ctx)->notify = notify_cb;
     (*ctx)->arg    = arg;
     (*ctx)->ev_base= base;
+    (*ctx)->first  = true;
 
     /* Success */
     status = SSTP_OKAY;
