@@ -20,21 +20,6 @@
  *  You should have received a copy of the GNU General Public License along
  *  with this program; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
- * @TODO:
- *   - Implement functions to get 
- *     -> sstp_stream_recv_http(), this receives a http response
- *     -> sstp_stream_recv_sstp(), this receives a sstp packet
- *   
- *   - We need to make sure we can send *and* receive sstp packets at 
- *     the same time, e.g. while sending; we may need to receive.
- *
- *   - Handle certificate verification, need to get the 
- *     certificate digest for use in the communication
- *     -> sstp_stream_certhash();   // Get certificate hash
- *     -> sstp_stream_getsess();    // Get SSL session info
- *
- *   - Set the SSL_MODE_AUTO_RETRY
  */
 
 #include <config.h>
@@ -180,7 +165,7 @@ static void sstp_operation_append(sstp_operation_st **head,
 }
 
 /*!
- * @brief Continue the send operation
+ * @brief Generic continue the send() operation
  */
 static void sstp_send_cont(int sock, short event, sstp_stream_st *ctx)
 {
@@ -206,12 +191,12 @@ static void sstp_send_cont(int sock, short event, sstp_stream_st *ctx)
 }
 
 /*! 
- * @brief Resume the send operation by retrying last operation
+ * @brief Generic resume the recv() operation by retrying last operation
  */
 static void sstp_recv_cont(int sock, short event, sstp_stream_st *ctx)
 {
     sstp_operation_st *op = &ctx->recv;
-    int ret = 0;
+    status_t ret;
 
     /* Handle Timeout */
     if (EV_TIMEOUT & event)
@@ -225,13 +210,9 @@ static void sstp_recv_cont(int sock, short event, sstp_stream_st *ctx)
             op->tout.tv_sec);
     if (ret == SSTP_INPROG)
         return;
-        
+
     /* Notify the caller of the status */
     op->complete(ctx, op->buf, op->arg, ret);
-
-    /* Re-add the event */
-    sstp_operation_add_read(ctx, op->buf, EV_READ,  
-            op->tout.tv_sec, op->complete, op->arg);
 }
 
 
@@ -286,6 +267,7 @@ done:
     return retval;
 }
 
+
 /*!
  * @brief Queue a write operation to the list of events
  */
@@ -377,6 +359,7 @@ done:
     return (status);
 }
 
+
 status_t sstp_verify_cert(sstp_stream_st *ctx, const char *host, int opts)
 {
     status_t status = SSTP_FAIL;
@@ -433,6 +416,7 @@ done:
     return status;
 }
 
+
 status_t sstp_last_activity(sstp_stream_st *stream, int seconds)
 {
     if (difftime(time(NULL), stream->last) > seconds)
@@ -444,16 +428,6 @@ status_t sstp_last_activity(sstp_stream_st *stream, int seconds)
 }
 
 
-/* 
- * Stubbed function for now...
- */
-status_t sstp_stream_recv_http(sstp_stream_st *ctx, sstp_buff_st *buf, 
-        sstp_complete_fn complete, void *arg, int timeout)
-{
-    return SSTP_NOTIMPL;
-}
-
-
 status_t sstp_stream_recv_plain(sstp_stream_st *ctx, sstp_buff_st *buf, 
         sstp_complete_fn complete, void *arg, int timeout)
 {
@@ -482,7 +456,8 @@ done:
     return status;
 }
 
-status_t sstp_stream_recv(sstp_stream_st *ctx, sstp_buff_st *buf, 
+
+status_t sstp_stream_recv_http(sstp_stream_st *ctx, sstp_buff_st *buf,
         sstp_complete_fn complete, void *arg, int timeout)
 {
     status_t status = SSTP_FAIL;
@@ -494,36 +469,40 @@ status_t sstp_stream_recv(sstp_stream_st *ctx, sstp_buff_st *buf,
     {
         event |= EV_TIMEOUT;
     }
-    ctx->recv_cb = sstp_stream_recv;
 
     /* Activity Timer */
     ctx->last = time(NULL);
 
-    /* Try to read from the SSL socket until it blocks */
-    ret = SSL_read(ctx->ssl, buf->data + buf->off, buf->max - buf->off);
-    switch (SSL_get_error(ctx->ssl, ret))
-    {
-    case SSL_ERROR_NONE:
-        buf->off += ret;
-        status = SSTP_OKAY;
-        break;
+    do {
+        /* Try to read from the SSL socket until it blocks */
+        ret = SSL_read(ctx->ssl, buf->data + buf->off, buf->max - buf->off);
+        switch (SSL_get_error(ctx->ssl, ret))
+        {
+        case SSL_ERROR_NONE:
+            buf->off += ret;
+            status = SSTP_OKAY;
+            break;
 
-    case SSL_ERROR_WANT_READ:
-        sstp_operation_add_read(ctx, buf, EV_READ, timeout,
-            complete, arg);
-        status = SSTP_INPROG;
-        goto done;
+        case SSL_ERROR_WANT_READ:
+            sstp_operation_add_read(ctx, buf, EV_READ, timeout,
+                complete, arg);
+            status = SSTP_INPROG;
+            goto done;
 
-    case SSL_ERROR_WANT_WRITE:
-        sstp_operation_add_read(ctx, buf, EV_WRITE, timeout,
-            complete, arg);
-        status = SSTP_INPROG;
-        goto done;
-    
-    default:
-        log_err("Unrecoverable SSL error %d", ret);
-        goto done;
-    }
+        case SSL_ERROR_WANT_WRITE:
+            sstp_operation_add_read(ctx, buf, EV_WRITE, timeout,
+                complete, arg);
+            status = SSTP_INPROG;
+            goto done;
+
+        default:
+            log_err("Unrecoverable SSL error %d", ret);
+            goto done;
+        }
+
+    // Read until we hit the end of the HTTP header
+    } while (buf->off < 2 ||
+            (buf->data[buf->off-1] != '\n' && buf->data[buf->off-0] != '\n'));
 
     status = SSTP_OKAY;
 
@@ -532,6 +511,7 @@ done:
     return status;
 }
 
+
 status_t sstp_stream_recv_sstp(sstp_stream_st *ctx, sstp_buff_st *buf, 
         sstp_complete_fn complete, void *arg, int timeout)
 {
@@ -584,17 +564,28 @@ done:
     return status;
 }
 
-void sstp_stream_setrecv(struct sstp_stream *ctx, sstp_recv_fn recv_cb,
+
+void sstp_stream_setrecv(sstp_stream_st *ctx, sstp_recv_fn recv_cb,
     sstp_buff_st *buf, sstp_complete_fn complete, void *arg, int timeout)
 {
     /* Setup the channel */
     ctx->recv_cb = recv_cb;
-    sstp_operation_add_read(ctx, buf, EV_READ, timeout, 
+    sstp_operation_add_read(ctx, buf, EV_READ, timeout,
             complete, arg);
     sstp_buff_reset(buf);
 }
 
 
+void sstp_stream_req_recv(sstp_stream_st *ctx)
+{
+    sstp_operation_st *op = &ctx->recv;
+
+    /* Re-add the event */
+    sstp_operation_add_read(ctx, op->buf, EV_READ,
+            op->tout.tv_sec, op->complete, op->arg);
+}
+
+
 /*!
  * @brief Continue the send operation
  */
@@ -716,6 +707,9 @@ status_t sstp_stream_send(sstp_stream_st *stream, sstp_buff_st *buf,
 }
 
 
+/**
+ * Configure the SSL object, setup socket and enable TLS SNI extenion if specified
+ */
 static status_t sstp_stream_setup(sstp_stream_st *stream)
 {
     sstp_option_st *opts = stream->opts;
@@ -760,6 +754,10 @@ done:
     return SSTP_FAIL;
 }
 
+
+/**
+ * Called when asynchronous connect() completes, i.e. socket becomes writable.
+ */
 static void sstp_connect_complete(int sock, short event, 
         sstp_stream_st *stream)
 {
@@ -795,6 +793,7 @@ done:
     op->complete(stream, NULL, op->arg, status);
 }
 
+
 status_t sstp_stream_connect(sstp_stream_st *stream, struct sockaddr *addr,
         int alen, sstp_complete_fn complete, void *arg, int timeout)
 {
